--[[
Name: Where's He Gone Now?
Revision: $Revision: 7 $
Author(s): ThePants999
Website: http://http://www.wowace.com/addons/wheres-he-gone-now/
SVN: svn://svn.wowace.com/wow/wheres-he-gone-now/mainline/trunk
Description: Keeps an eye on someone's distance from you, and alerts you if they get too far.
License: GNU General Public License version 3 (GPLv3)
]]

-- State

local WarningActive = false



-- Initialization

local AppName = "Where's He Gone Now?"
local VERSION = AppName .. " -r" .. ("$Revision: 7 $"):match("%d+")

local rc = LibStub("LibRangeCheck-2.0")
WHGN = LibStub("AceAddon-3.0"):NewAddon(AppName, "AceConsole-3.0", "AceTimer-3.0")



-- Config functions

function WHGN:GetEnabled(info)
  return self.db.char.Enabled
end

function WHGN:SetEnabled(info, input)
  if input then
    self.db.char.Enabled = true
  else
    self.db.char.Enabled = false
  end
  
  if self.db.char.Enabled then
    self:OnEnable(false)
  else
    self:OnDisable()
  end
end

function WHGN:GetInterval(info)
  return self.db.char.UpdateInterval
end

function WHGN:SetInterval(info, input)
  self.db.char.UpdateInterval = input
  self:OnDisable()
  self:OnEnable(false)
end

function WHGN:GetChar(info)
  return self.db.char.PlayerName
end

function WHGN:SetChar(info, input)
  self.db.char.PlayerName = input
  WarningActive = false
end

function WHGN:GetRange(info)
  return self.db.char.MaxRange
end

function WHGN:SetRange(info, input)
  self.db.char.MaxRange = input
  WarningActive = false
end




-- Config options

local OptionsTable = {
  handler = WHGN,
  type = "group",
  args = {
    enable = {
      name = "Enabled",
      desc = "Enable or disable the addon.",
      type = "toggle",
      get = "GetEnabled",
      set = "SetEnabled",
    },
    interval = {
      name = "Update interval",
      desc = "How often should the addon check the range? Lower values mean you find out faster if they're on the wander, but the addon uses more CPU.",
      type = "range",
      min = 0.1,
      max = 3.0,
      step = 0.1,
      get = "GetInterval",
      set = "SetInterval",
    },
    char = {
      name = "Character name",
      desc = "Which character do you want to track?",
      type = "input",
      get = "GetChar",
      set = "SetChar",
    },
    range = {
      name = "Maximum range",
      desc = "How far away will you allow them to get before you get a warning?",
      type = "range",
      min = 20,
      max = 79,
      step = 1,
      get = "GetRange",
      set = "SetRange",
    },
  }
}

LibStub("AceConfig-3.0"):RegisterOptionsTable(AppName, OptionsTable, {})
local OptionsDialog = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AppName, AppName)

function WHGN:ShowOptionsDialog()
  InterfaceOptionsFrame_OpenToCategory(OptionsDialog)
end

-- AceAddon functions

function WHGN:OnInitialize()
  -- Set defaults
  local defaults = {
    char = {
      Enabled = true,
      PlayerName = "Daftagnan",
      MaxRange = 55,
      UpdateInterval = 0.5,
    }
  }
      
  self.db = LibStub("AceDB-3.0"):New("WHGN_DB", defaults)  
  self:Print("Tracking " .. self.db.char.PlayerName .. ", maximum range " .. self.db.char.MaxRange)
end

function WHGN:OnEnable(first)
  self:RegisterChatCommand("whgn", "ShowOptionsDialog")
  if self.db.char.Enabled then
    self.timer = self:ScheduleRepeatingTimer("TimerPop", self.db.char.UpdateInterval)
  end
  WarningActive = false  
end

function WHGN:OnDisable()
  self:UnregisterChatCommand("whgn")
  self:CancelAllTimers()
end



-- Real functionality!

function WHGN:TimerPop()
  if WarningActive then
    -- We've already warned the player. Check to see if we're OK again.
    if (UnitInParty(self.db.char.PlayerName) or UnitInRaid(self.db.char.PlayerName)) and (UnitAffectingCombat("player") or UnitAffectingCombat(self.db.char.PlayerName)) then
      -- Still in combat; range is relevant.
      local min, max = rc:GetRange(self.db.char.PlayerName)
      local range = 0
      
      if min and max then
        range = (min + max) / 2
      elseif max then
        range = max
      elseif min then
        -- We have no maximum. Must be pretty huge.
        range = 100
      end
      
      if range <= self.db.char.MaxRange then
        -- Back within range, phew.
        WarningActive = false
      end
    else
      -- No longer in combat, or no longer grouped.
      WarningActive = false
    end
  else
    -- Warning currently inactive. Do we now have a problem?
    
    if (UnitInParty(self.db.char.PlayerName) or UnitInRaid(self.db.char.PlayerName)) and (UnitAffectingCombat("player") or UnitAffectingCombat(self.db.char.PlayerName)) then
      -- In combat; range is relevant.
      local min, max = rc:GetRange(self.db.char.PlayerName)
      local range = 0
      
      if min and max then
        range = (min + max) / 2
      elseif max then
        range = max
      elseif min then
        -- We have no maximum. Must be pretty huge.
        range = 100
      end
      
      if range > self.db.char.MaxRange then
        -- Noooo! Out of range!
        self:Print(self.db.char.PlayerName .. " is getting away!")
        PlaySound("TellMessage")
        WarningActive = true
      end
    end
  end
end